<?php
/**
 * Emailer - PHP Emailer creation class.
 * PHP Version 5.3+
 * @package Emailer
 * @author Pawel Klopotowski <webmaster@pklopotowski.pl>
 * @copyright 2016
 * @license MIT
 * @note:
 * You are using this software at your own risk. 
 * Any liability associated with the misuse of this software is on your side. 
 */
class Emailer
{
    /**
     * The Cms Version number.
     * @var string
     */
    public $version = '1.16';

    /**
     * Component name to be proceed.
     * @var string
     */
    public $manage;

    /**
     * Action to be fired on the component.
     * @var string
     */
    public $action;

    /**
     * Data type.
     * @var string
     */
    public $data;

    /**
     * Tracking email flag. If true view will not be loaded.
     * @var string
     */
    public $track;

    /**
     * Subscribtion flag. If true view will not be loaded.
     * @var boolean
     */
    public $subscribe;

    /**
     * Subscribtion confirmation flag. If true view will not be loaded.
     * @var boolean
     */
    public $subscribeConfirm;

    /**
     * Unsubscribtion flag. If true view will not be loaded.
     * @var boolean
     */
    public $unsubscribe;

    /**
     * Web browser flag. If true view will not be loaded.
     * @var boolean
     */
    public $browserview;

    /**
     * Stop background process flag.
     * @var boolean
     */
    public $stop;

    /**
     * Click flag. If true view will not be loaded.
     * @var boolean
     */
    public $click;

    /**
     * Login flag. If true redirect to login page.
     * @var boolean
     */
    public $login;

    /**
     * Logout flag. If true redirect to logout page.
     * @var boolean
     */
    public $logout;

    /**
     * Run flag. Determine is the program working or not.
     * @var boolean
     */
    public $run;

    /**
     * Menu flag. Not used yet.
     * @var boolean
     */
    public $menu;

    /**
     * Component name to load.
     * @var string
     */
    public $component;

    /**
     * Mailing working directory.
     * @var string
     */
    public $mailing_directory;

    /**
     * CSRF token.
     * @var string
     * @access private
     */
    private $tokenCSRF;

    /**
     * Constructor.
     * @params setup
     */
    function __construct()
    {
        $this->manage = isset($_GET['manage']) ? $_GET['manage'] : 'default';
        $this->action = isset($_GET['action']) ? $_GET['action'] : 'default';
        $this->data = isset($_GET['data']) ? $_GET['data'] : 'default';
        $this->track = $this->action == 'track' ? true : false;
        $this->subscribe = $this->action == 'subscribe' ? true : false;
        $this->subscribeConfirm = $this->action == 'subscribe.confirmed' ? true : false;
        $this->unsubscribe = $this->action == 'unsubscribe' ? true : false;
        $this->browserview = $this->action == 'browserview' ? true : false;
        $this->stop = $this->action == 'stop.background' ? true : false;
        $this->click = $this->action == 'click' ? true : false;
        $this->login = empty($_SESSION['admin']) ? true : false;
        $this->logout = $this->action == 'logout' ? true : false;
        $this->run = true;
        $this->menu = false;
        $this->component = ($this->data != 'default' || @$_POST['adata']) ? true : false;
        $this->tokenCSRF = isset($_SESSION['csrf_token']) ? $_SESSION['csrf_token'] : false;
        $this->mailing_directory = dirname($_SERVER['SCRIPT_NAME']);
    }
    
    /**
     * Check posted CSRF token. It must be the same as stored in the session.
     * md5 is added to hash posted value, just in case of insecure values.
     * @access private
     * @return boolean
     */
    private function isSecureCSRF()
    {
        return (md5($this->tokenCSRF) == md5($_POST['csrf_token'])) ? true : false;
    }
    
    /**
     * Return CSRF token.
     * @return string
     */
    public function getToken()
    {
        return $this->tokenCSRF;
    }
    
    /**
     * Check security of CSRF token. Stop the program if it is wrong.
     * @return boolean or die
     */
    public function checkToken()
    {
        return $this->isSecureCSRF() ? true : die("access denied");
    }

    /**
     * Load system language.
     * @return void
     */
    public function loadLanguage()
    {
        include("languages/".Settings::$systemLanguage.".php");
    }

    /**
     * Store system params in configuration.php file.
     * @param string $param. The param to be changed.
     * @param string $value. Value to be set.
     * @param string $method. Secure Hash Algorithm to be used:
     * * `sha1` Calculate the SHA-1 hash of the string;
     * * `md5` Calculate the MD5 hash of a string.
     * @return void
     * @access public
     */
    public function changeSetting($param, $value, $method='default')
    {
        if ($method == 'sha1') {
            $value = sha1($value);
        }
        if ($method == 'md5') {
            $value = md5($value);
        }
        $fileSettings = $_SERVER['DOCUMENT_ROOT'].dirname($_SERVER['SCRIPT_NAME']).DIRECTORY_SEPARATOR.'configuration.php';
        $fileContent = file_get_contents($fileSettings);
        $fileNewContent = preg_replace("/$param = \'(.*)\'/i", "$param = '".$value."'", $fileContent);
        file_put_contents($fileSettings, $fileNewContent);
    }

    /**
     * Load login page.
     * @return void
     */
    public function login()
    {
        include("php/login.php");
        exit();
    }

    /**
     * Proceed logout and restart system.
     * @return void
     */
    public function logout()
    {
        $_SESSION['admin'] = false;
        $_SESSION['csrf_token'] = false;
        session_destroy();
        echo "<script> window.location.replace('index.php');</script>";
    }
    
    /**
     * Clear string to avoid SQL injection.
     * @return string
     */
    public function clearSQL($data)
    {
        $data = preg_replace("/[;\'\"\\*]/", "", $data);
        return $data;
    }

}
